﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using PDFTron.SilverDox.Samples.SubControls;
using PDFTron.SilverDox.Controls;

namespace PDFTron.SilverDox.Samples
{
    /// <summary>
    /// Represents a control that contains the outline treeview and thumbnail listbox controls
    /// </summary>
	public partial class SideWindowControl : UserControl
	{
		
        /// <summary>
        /// Creates a new instance of the SideWindowControl
        /// </summary>
		public SideWindowControl()
		{			
			InitializeComponent();
		}
        
        /// <summary>
        /// Creates a side window based on customization options from ReaderControl
        /// </summary>
        /// <param name="docViewer">a ReaderControl that holds customization options </param>
		public void CreateSideWindow(ReaderControl docViewer)
		{
			if (docViewer.EnableOutlineTreeControl)
			{
				OutlineTreeControl outlineTree = new OutlineTreeControl();
				this.OutlineTabItem.Content = outlineTree;
				this.OutlineTabItem.Visibility = Visibility.Visible;                
			}
		
			if (docViewer.EnableThumbnailListControl)
			{                
				ThumbnailsControl thumbnailViewer = new ThumbnailsControl()
				{                    
					Background = new SolidColorBrush(Colors.White),
					Foreground = new SolidColorBrush(Colors.Black),
                    ScaleFactor = 1,										
				};
				
				this.ThumbnailTabItem.Content = thumbnailViewer;
				this.ThumbnailTabItem.Visibility = Visibility.Visible;

				if (!docViewer.EnableOutlineTreeControl)
					this.ThumbnailTabItem.IsSelected = true;
			}

            if (docViewer.EnableWholeDocumentSearchControl)
            {
                WholeDocumentSearchControl wholeDocumentViewerControl = new WholeDocumentSearchControl();

                this.SearchTabItem.Visibility = Visibility.Visible;
                this.SearchTabItem.Content = wholeDocumentViewerControl;
            }


		}
	}
}