﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using PDFTron.SilverDox.Controls;

using PDFTron.SilverDox.Samples.SubControls;



namespace PDFTron.SilverDox.Samples
{
    /// <summary>
    /// Represents a customizable toolbar for manipulating a DocumentViewer.
    /// The DataContext of this control must be a DocumentViewer.
    /// </summary>
	public partial class ToolbarControl : UserControl
	{

        /// <summary>
        /// Creates a new instance of the ToolbarControl
        /// </summary>
		public ToolbarControl()
		{
			InitializeComponent();
		}

        /// <summary>
        /// Creates a toolbar based on customization options from ReaderControl
        /// </summary>
        /// <param name="reader">a ReaderControl that holds customization options </param>
		public void CreateToolbar(ReaderControl reader)
		{

			// first group of tools: outline toggle 
			if (reader.EnableOutlineToggleControl)
			{				
				this.LeftToolStackPanel.Children.Add(new OutlineToggleButtonControl(reader));				
			}

			int toolsLeft = CountNumberOfTools(reader);
			int i = 0;

			if (reader.EnableOpenLocalFileControl)
			{
				this.ToolStackPanel.Children.Add(new OpenLocalFileButtonControl(reader));
				i++;
			}

			// add divider if necessary
			toolsLeft = toolsLeft - i;
			if (i > 0 && toolsLeft > 0)
			{
				this.ToolStackPanel.Children.Add(new DividerControl());
				i = 0;
			}

			// next group of tools: Page 
			if (reader.EnablePageNumberControl)
			{
				this.ToolStackPanel.Children.Add(new PageNumberControl());
				i++;
            }

            // add divider if necessary
            toolsLeft = toolsLeft - i;
            if (i > 0 && toolsLeft > 0)
            {
                this.ToolStackPanel.Children.Add(new DividerControl());
                i = 0;
            }

            if (reader.EnableLayoutControl)
            {
                this.ToolStackPanel.Children.Add(new LayoutControl(reader));
                i++;
            }

            // add divider if necessary
            toolsLeft = toolsLeft - i;
            if (i > 0 && toolsLeft > 0)
            {
                this.ToolStackPanel.Children.Add(new DividerControl());
                i = 0;
            }

            if (reader.EnableRotateControl)
            {
                this.ToolStackPanel.Children.Add(new RotatePagesControl());
                i++;
            }

            // add divider if necessary
            toolsLeft = toolsLeft - i;
            if (i > 0 && toolsLeft > 0)
            {
                this.ToolStackPanel.Children.Add(new DividerControl());
                i = 0;
            }

			if (reader.EnablePageNavigationControl)
			{
				this.ToolStackPanel.Children.Add(new PageNavigationControl());
				i++;
			}

			// add divider if necessary
			toolsLeft = toolsLeft - i;
			if (i > 0 && toolsLeft > 0)
			{
				this.ToolStackPanel.Children.Add(new DividerControl());
				i = 0;
			}

			// next group of tools: Zoom
			if (reader.EnableZoomSliderControl)
			{
				this.ToolStackPanel.Children.Add(new ZoomSliderControl());
				i++;
			}

			if (reader.EnableZoomTextBoxControl)
			{
				this.ToolStackPanel.Children.Add(new ZoomTextBoxControl());
				i++;
			}

			// add divider if necessary
			toolsLeft = toolsLeft - i;
			if (i > 0 && toolsLeft > 0)
			{
				this.ToolStackPanel.Children.Add(new DividerControl());
				i = 0;
			}

            // next group of tools: Fit Mode
            if (reader.EnableFitModeControl)
            {
                this.ToolStackPanel.Children.Add(new FitModeControl(reader));
                i++;
            }

            // add divider if necessary
            toolsLeft = toolsLeft - i;
            if (i > 0 && toolsLeft > 0)
            {
                this.ToolStackPanel.Children.Add(new DividerControl());
                i = 0;
            }

			// next group of tools: ToolMode
			if (reader.EnableToolModeControl)
			{
				this.ToolStackPanel.Children.Add(new ToolModeControl());
				i++;
			}

			// add divider if necessary
			toolsLeft = toolsLeft - i;
			if (i > 0 && toolsLeft > 0)
			{
				this.ToolStackPanel.Children.Add(new DividerControl());
				i = 0;
			}

			// next group of tools: Search
			if (reader.EnableSearchControl)
			{
				this.ToolStackPanel.Children.Add(new SearchControl());
				i++;
			}

			// add divider if necessary
			toolsLeft = toolsLeft - i;
			if (i > 0 && toolsLeft > 0)
			{
				this.ToolStackPanel.Children.Add(new DividerControl());
				i = 0;
			}

			// last group of tools: Auxilary            
			if (reader.EnablePrintControl)
			{
				this.ToolStackPanel.Children.Add(new PrintButtonControl());
			}
			if (reader.EnableFullScreenControl)
			{
				this.ToolStackPanel.Children.Add(new FullScreenButtonControl());
			}



		}

		private void ZoomLevelBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
		{
			DocumentViewer fixedDocumentViewer = ((ComboBox)sender).DataContext as DocumentViewer;
			if (fixedDocumentViewer != null)
			{
				double zoomLevel = 0;
				if (double.TryParse((string)(e.AddedItems[0] as ComboBoxItem).Tag, out zoomLevel))
				{
					fixedDocumentViewer.Zoom = zoomLevel;
				}
			}
		}

		private int CountNumberOfTools(ReaderControl s)
		{
			int i = 0;
			
            if (s.EnablePageNumberControl) i++;
			if (s.EnablePageNavigationControl) i++;            
			if (s.EnableZoomSliderControl) i++;
            if (s.EnableZoomTextBoxControl) i++;
            if (s.EnableOpenLocalFileControl) i++;
            if (s.EnableFitModeControl) i++;
			if (s.EnableToolModeControl) i++;
			if (s.EnablePrintControl) i++;
			if (s.EnableFullScreenControl) i++;			
			
			return i;
		}
	}
}
